# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

from . import types
from . import stream_types


type_map = {

    "types.CapabilityCandidate": types.CapabilityCandidate,
    "stream_types.CapabilityCandidate": stream_types.CapabilityCandidate,

    "types.CapabilityCategory": types.CapabilityCategory,
    "stream_types.CapabilityCategory": stream_types.CapabilityCategory,

    "types.CapabilityMatchResult": types.CapabilityMatchResult,
    "stream_types.CapabilityMatchResult": stream_types.CapabilityMatchResult,

    "types.CapabilitySummary": types.CapabilitySummary,
    "stream_types.CapabilitySummary": stream_types.CapabilitySummary,

    "types.CodeSnippet": types.CodeSnippet,
    "stream_types.CodeSnippet": stream_types.CodeSnippet,

    "types.ManifestCLI": types.ManifestCLI,
    "stream_types.ManifestCLI": stream_types.ManifestCLI,

    "types.ManifestServer": types.ManifestServer,
    "stream_types.ManifestServer": stream_types.ManifestServer,

    "types.ManifestSummary": types.ManifestSummary,
    "stream_types.ManifestSummary": stream_types.ManifestSummary,

    "types.ToolArg": types.ToolArg,
    "stream_types.ToolArg": stream_types.ToolArg,

    "types.ToolDescription": types.ToolDescription,
    "stream_types.ToolDescription": stream_types.ToolDescription,

    "types.ToolInfo": types.ToolInfo,
    "stream_types.ToolInfo": stream_types.ToolInfo,


}