# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

import typing
import typing_extensions
import baml_py

from . import stream_types, types, type_builder
from .parser import LlmResponseParser, LlmStreamParser
from .runtime import DoNotUseDirectlyCallManager, BamlCallOptions
from .globals import DO_NOT_USE_DIRECTLY_UNLESS_YOU_KNOW_WHAT_YOURE_DOING_RUNTIME as __runtime__


class BamlAsyncClient:
    __options: DoNotUseDirectlyCallManager
    __stream_client: "BamlStreamClient"
    __http_request: "BamlHttpRequestClient"
    __http_stream_request: "BamlHttpStreamRequestClient"
    __llm_response_parser: LlmResponseParser
    __llm_stream_parser: LlmStreamParser

    def __init__(self, options: DoNotUseDirectlyCallManager):
        self.__options = options
        self.__stream_client = BamlStreamClient(options)
        self.__http_request = BamlHttpRequestClient(options)
        self.__http_stream_request = BamlHttpStreamRequestClient(options)
        self.__llm_response_parser = LlmResponseParser(options)
        self.__llm_stream_parser = LlmStreamParser(options)

    def with_options(self,
        tb: typing.Optional[type_builder.TypeBuilder] = None,
        client_registry: typing.Optional[baml_py.baml_py.ClientRegistry] = None,
        client: typing.Optional[str] = None,
        collector: typing.Optional[typing.Union[baml_py.baml_py.Collector, typing.List[baml_py.baml_py.Collector]]] = None,
        env: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        tags: typing.Optional[typing.Dict[str, str]] = None,
        on_tick: typing.Optional[typing.Callable[[str, baml_py.baml_py.FunctionLog], None]] = None,
    ) -> "BamlAsyncClient":
        options: BamlCallOptions = {}
        if tb is not None:
            options["tb"] = tb
        if client_registry is not None:
            options["client_registry"] = client_registry
        if client is not None:
            options["client"] = client
        if collector is not None:
            options["collector"] = collector
        if env is not None:
            options["env"] = env
        if tags is not None:
            options["tags"] = tags
        if on_tick is not None:
            options["on_tick"] = on_tick
        return BamlAsyncClient(self.__options.merge_options(options))

    @property
    def stream(self):
      return self.__stream_client

    @property
    def request(self):
      return self.__http_request

    @property
    def stream_request(self):
      return self.__http_stream_request

    @property
    def parse(self):
      return self.__llm_response_parser

    @property
    def parse_stream(self):
      return self.__llm_stream_parser

    async def GenerateCodeSnippet(self, tool: types.ToolInfo,
        baml_options: BamlCallOptions = {},
    ) -> types.CodeSnippet:
        # Check if on_tick is provided
        if 'on_tick' in baml_options:
            # Use streaming internally when on_tick is provided
            __stream__ = self.stream.GenerateCodeSnippet(tool=tool,
                baml_options=baml_options)
            return await __stream__.get_final_response()
        else:
            # Original non-streaming code
            __result__ = await self.__options.merge_options(baml_options).call_function_async(function_name="GenerateCodeSnippet", args={
                "tool": tool,
            })
            return typing.cast(types.CodeSnippet, __result__.cast_to(types, types, stream_types, False, __runtime__))
    async def MatchCapability(self, query: str,manifest: types.ManifestSummary,available_clis: typing.List[str],running_servers: typing.List[str],
        baml_options: BamlCallOptions = {},
    ) -> types.CapabilityMatchResult:
        # Check if on_tick is provided
        if 'on_tick' in baml_options:
            # Use streaming internally when on_tick is provided
            __stream__ = self.stream.MatchCapability(query=query,manifest=manifest,available_clis=available_clis,running_servers=running_servers,
                baml_options=baml_options)
            return await __stream__.get_final_response()
        else:
            # Original non-streaming code
            __result__ = await self.__options.merge_options(baml_options).call_function_async(function_name="MatchCapability", args={
                "query": query,"manifest": manifest,"available_clis": available_clis,"running_servers": running_servers,
            })
            return typing.cast(types.CapabilityMatchResult, __result__.cast_to(types, types, stream_types, False, __runtime__))
    async def SummarizeCapabilities(self, tools: typing.List["types.ToolDescription"],
        baml_options: BamlCallOptions = {},
    ) -> types.CapabilitySummary:
        # Check if on_tick is provided
        if 'on_tick' in baml_options:
            # Use streaming internally when on_tick is provided
            __stream__ = self.stream.SummarizeCapabilities(tools=tools,
                baml_options=baml_options)
            return await __stream__.get_final_response()
        else:
            # Original non-streaming code
            __result__ = await self.__options.merge_options(baml_options).call_function_async(function_name="SummarizeCapabilities", args={
                "tools": tools,
            })
            return typing.cast(types.CapabilitySummary, __result__.cast_to(types, types, stream_types, False, __runtime__))
    


class BamlStreamClient:
    __options: DoNotUseDirectlyCallManager

    def __init__(self, options: DoNotUseDirectlyCallManager):
        self.__options = options

    def GenerateCodeSnippet(self, tool: types.ToolInfo,
        baml_options: BamlCallOptions = {},
    ) -> baml_py.BamlStream[stream_types.CodeSnippet, types.CodeSnippet]:
        __ctx__, __result__ = self.__options.merge_options(baml_options).create_async_stream(function_name="GenerateCodeSnippet", args={
            "tool": tool,
        })
        return baml_py.BamlStream[stream_types.CodeSnippet, types.CodeSnippet](
          __result__,
          lambda x: typing.cast(stream_types.CodeSnippet, x.cast_to(types, types, stream_types, True, __runtime__)),
          lambda x: typing.cast(types.CodeSnippet, x.cast_to(types, types, stream_types, False, __runtime__)),
          __ctx__,
        )
    def MatchCapability(self, query: str,manifest: types.ManifestSummary,available_clis: typing.List[str],running_servers: typing.List[str],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.BamlStream[stream_types.CapabilityMatchResult, types.CapabilityMatchResult]:
        __ctx__, __result__ = self.__options.merge_options(baml_options).create_async_stream(function_name="MatchCapability", args={
            "query": query,"manifest": manifest,"available_clis": available_clis,"running_servers": running_servers,
        })
        return baml_py.BamlStream[stream_types.CapabilityMatchResult, types.CapabilityMatchResult](
          __result__,
          lambda x: typing.cast(stream_types.CapabilityMatchResult, x.cast_to(types, types, stream_types, True, __runtime__)),
          lambda x: typing.cast(types.CapabilityMatchResult, x.cast_to(types, types, stream_types, False, __runtime__)),
          __ctx__,
        )
    def SummarizeCapabilities(self, tools: typing.List["types.ToolDescription"],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.BamlStream[stream_types.CapabilitySummary, types.CapabilitySummary]:
        __ctx__, __result__ = self.__options.merge_options(baml_options).create_async_stream(function_name="SummarizeCapabilities", args={
            "tools": tools,
        })
        return baml_py.BamlStream[stream_types.CapabilitySummary, types.CapabilitySummary](
          __result__,
          lambda x: typing.cast(stream_types.CapabilitySummary, x.cast_to(types, types, stream_types, True, __runtime__)),
          lambda x: typing.cast(types.CapabilitySummary, x.cast_to(types, types, stream_types, False, __runtime__)),
          __ctx__,
        )
    

class BamlHttpRequestClient:
    __options: DoNotUseDirectlyCallManager

    def __init__(self, options: DoNotUseDirectlyCallManager):
        self.__options = options

    async def GenerateCodeSnippet(self, tool: types.ToolInfo,
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="GenerateCodeSnippet", args={
            "tool": tool,
        }, mode="request")
        return __result__
    async def MatchCapability(self, query: str,manifest: types.ManifestSummary,available_clis: typing.List[str],running_servers: typing.List[str],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="MatchCapability", args={
            "query": query,"manifest": manifest,"available_clis": available_clis,"running_servers": running_servers,
        }, mode="request")
        return __result__
    async def SummarizeCapabilities(self, tools: typing.List["types.ToolDescription"],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="SummarizeCapabilities", args={
            "tools": tools,
        }, mode="request")
        return __result__
    

class BamlHttpStreamRequestClient:
    __options: DoNotUseDirectlyCallManager

    def __init__(self, options: DoNotUseDirectlyCallManager):
        self.__options = options

    async def GenerateCodeSnippet(self, tool: types.ToolInfo,
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="GenerateCodeSnippet", args={
            "tool": tool,
        }, mode="stream")
        return __result__
    async def MatchCapability(self, query: str,manifest: types.ManifestSummary,available_clis: typing.List[str],running_servers: typing.List[str],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="MatchCapability", args={
            "query": query,"manifest": manifest,"available_clis": available_clis,"running_servers": running_servers,
        }, mode="stream")
        return __result__
    async def SummarizeCapabilities(self, tools: typing.List["types.ToolDescription"],
        baml_options: BamlCallOptions = {},
    ) -> baml_py.baml_py.HTTPRequest:
        __result__ = await self.__options.merge_options(baml_options).create_http_request_async(function_name="SummarizeCapabilities", args={
            "tools": tools,
        }, mode="stream")
        return __result__
    

b = BamlAsyncClient(DoNotUseDirectlyCallManager({}))